# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - MoinMessage library

    @copyright: 2012 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from email import message_from_string
from email.encoders import encode_noop
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication
from email.mime.base import MIMEBase
from email.mime.text import MIMEText
from subprocess import Popen, PIPE
from tempfile import mkstemp
import httplib
import os

class Message:

    "An update message."

    def __init__(self):
        self.updates = []

    def add_update(self, alternatives):
        if len(alternatives) > 1:
            part = MIMEMultipart()
            for alternative in alternatives:
                part.attach(alternative)
            self.updates.append(part)
        else:
            self.updates.append(alternatives[0])

    def get_payload(self):
        if len(self.updates) == 1:
            message = self.updates[0]
        else:
            message = MIMEMultipart()
            message.add_header("Update-Type", "collection")
            for update in self.updates:
                message.attach(update)

        return message

class MoinMessageError(Exception):
    pass

class GPG:

    "A wrapper around the gpg command using a particular configuration."

    def __init__(self, homedir=None):
        self.conf_args = []

        if homedir:
            self.conf_args += ["--homedir", homedir]

        self.errors = None

    def run(self, args, text=None):

        """
        Invoke gpg with the given 'args', supplying the given 'text' to the
        command directly or, if 'text' is omitted, using a file provided as part
        of the 'args' if appropriate.

        Failure to complete the operation will result in a MoinMessageError
        being raised.
        """

        cmd = Popen(["gpg"] + self.conf_args + list(args), stdin=PIPE, stdout=PIPE, stderr=PIPE)

        if text:
            cmd.stdin.write(text)
        cmd.stdin.close()

        self.errors = cmd.stderr.read()

        try:
            text = cmd.stdout.read()

            # Test for a zero result.

            if not cmd.wait():
                return text
            else:
                raise MoinMessageError, errors

        finally:
            cmd.stdout.close()
            cmd.stderr.close()

    def verifyMessage(self, signature, content):

        "Using the given 'signature', verify the given message 'content'."

        # Write the detached signature and content to files.

        signature_fd, signature_filename = mkstemp()
        content_fd, content_filename = mkstemp()

        try:
            signature_fp = os.fdopen(signature_fd, "w")
            content_fp = os.fdopen(content_fd, "w")
            try:
                signature_fp.write(signature)
                content_fp.write(content)
            finally:
                signature_fp.close()
                content_fp.close()

            # Verify the message text.

            self.run(["--verify", signature_filename, content_filename])

        finally:
            os.remove(signature_filename)
            os.remove(content_filename)

    def signMessage(self, message, keyid):

        """
        Return a signed version of 'message' using the given 'keyid'.
        """

        text = message.as_string()
        signature = self.run(["--armor", "-u", keyid, "--detach-sig"], text)

        # Make the container for the message.

        signed_message = MIMEMultipart("signed", protocol="application/pgp-signature")
        signed_message.attach(message)

        signature_part = MIMEBase("application", "pgp-signature")
        signature_part.set_payload(signature)
        signed_message.attach(signature_part)

        return signed_message

    def decryptMessage(self, message):

        "Return a decrypted version of 'message'."

        return self.run(["--decrypt"], message)

    def encryptMessage(self, message, keyid):

        """
        Return an encrypted version of 'message' using the given 'keyid'.
        """

        text = message.as_string()
        encrypted = self.run(["--armor", "-r", keyid, "--encrypt", "--trust-model", "always"], text)

        # Make the container for the message.

        encrypted_message = MIMEMultipart("encrypted", protocol="application/pgp-encrypted")

        # For encrypted content, add the declaration and content.

        declaration = MIMEBase("application", "pgp-encrypted")
        declaration.set_payload("Version: 1")
        encrypted_message.attach(declaration)

        content = MIMEApplication(encrypted, "octet-stream", encode_noop)
        encrypted_message.attach(content)

        return encrypted_message

# Communications functions.

def sendMessage(message, host, path):

    "Send 'message' to the given 'host' using the specified URL 'path'."

    text = message.as_string()

    req = httplib.HTTPConnection(host)
    req.request("PUT", path, text) # {"Content-Length" : len(text)}
    resp = req.getresponse()
    return resp.read()

# vim: tabstop=4 expandtab shiftwidth=4
